%% DANIEL MORENO BOZA, MAE210C. THE RAYLEIGH-BENARD PROBLEM 

% This file computes the marginal curve Ra = Ra(k) using chebfun. The sixth
% order problem for the velocity is considered and spectral collocation is
% used for discretizing the differential operators.
% ARPACK is used for solving the generalized eigenvalue problem.

%%

close all
clear all

% This line turned out to be crucial to get a good result for Ra (high
% order eigenvalue problems may lead to badly-conditioned matrices)

cheboppref.setDefaults('discretization', 'ultraspherical')

%%

nev = 2;

% kv  = 2.682;
kv  = linspace(0.5,8,50);
Ra0 = 6000;

%%

figure(1); hold on
xlabel('$\tilde{k}$','fontsize',20,'interpreter','latex');
ylabel('$Ra$','fontsize',20,'interpreter','latex')
axis([0 kv(end) 0 8000]);

%% Initialize operators and iterate on the wavenumber vector

A = chebop(0,1);
B = chebop(0,1);

for jj = 1:length(kv)
    
    k = kv(jj);    
    
    % Define operators as you read them from the equations
    
    A.op = @(x,u) ( diff(u,6) - 3*k^2*diff(u,4) + 3*k^4*diff(u,2) - k^6*u );
    B.op = @(x,u) ( -k^2.*u );
    
    % Impose boundary conditions in A
    
    A.lbc =  @(u) [u ; diff(u,1); diff(u,4) - 2*k^2*diff(u,2)];
    A.rbc =  @(u) [u ; diff(u,2); diff(u,4)                  ];
    
    % Solve the eigenvalue problem
    
    [V,D] = eigs(A,B,nev,Ra0);    
   
    [Rac,ind] = min(diag(D));
    
    plot(k,Rac,'k.'); drawnow;
    
    Ra0 = Rac;
    Rs(jj) = Rac;
    
end

%% Save results

file_name = 'Rac_cb1';
save(file_name,'kv','Rs');



