%% DANIEL MORENO BOZA, MAE210C. THE RAYLEIGH-BENARD PROBLEM

% This file computes the marginal curve Ra = Ra(k) using chebfun. The
% fourth order problem for the velocity and second order problem for the
% temperature is considered and spectral collocation is used for
% discretizating the differential operators.
% ARPACK is used for solving the generalized eigenvalue problem.

%%

clear all
close all

% This line turned out to be crucial to get a good result for Ra (high
% order eigenvalue problems may lead to badly-conditioned matrices)

cheboppref.setDefaults('discretization', 'coeffs')

%%

nev = 2;

kv = 2.682;
% kv  = linspace(0.5,8,50);
Ra0 = 6000;

casestudy = 'freesolid';

%%

figure(1); hold on
xlabel('$\tilde{k}$','fontsize',20,'interpreter','latex');
ylabel('$Ra$','fontsize',20,'interpreter','latex')
axis([0 kv(end) 0 8000]);

%% Initialize operators and iterate on the wavenumber vector

A  = chebop(0,1);
B  = chebop(0,1);

for jj = 1:length(kv)
    
    k = kv(jj);
    
    % Initialize operators. In this case, every operator contains two rows
    % because we have two dependent variables
    
    A.op  = @(z,W,T) [
        diff(W,4) - 2*k^2*diff(W,2) + k^4*W - k^2*T;
        diff(T,2) -   k^2*T
        ];
    
    B.op  = @(z,W,T) [0*T; -W];
    
    % Impose boundary conditions in A
    
    switch casestudy
        
        case 'freefree'
            
            A.lbc = @(W,T) [W;  diff(W,2); T];
            A.rbc = @(W,T) [W;  diff(W,2); T]; 
                        
        case 'freesolid'
            
            A.lbc = @(W,T) [W;  diff(W)  ; T];
            A.rbc = @(W,T) [W;  diff(W,2); T];
                        
        case 'solidsolid'
            
            A.lbc = @(W,T) [W;  diff(W)  ; T];
            A.rbc = @(W,T) [W;  diff(W)  ; T];
            
    end
    
    % Solve the eigenvalue problem
    
    [V,D] = eigs(A,B,nev,Ra0);
    
    W = V(:,1);
    T = V(:,2);
    
    [Rac,pos]   = min(diag(D));
    W           = W{pos};
    T           = T{pos};
    
    plot(k,Rac,'.'); drawnow
    
    Ra0    = Rac;
    Rs(jj) = Rac;
    
end

%% Save results

file_name = 'Rac_cb2';
save(file_name,'kv','Rs');
