%% DANIEL MORENO BOZA, MAE210C. THE RAYLEIGH-BENARD PROBLEM

% This file computes the marginal curve Ra = Ra(k) using FDM. The sixth
% order problem for the velocity is considered and a centered-2nd order-fdm
% scheme is used for the discretization of the differential operators.
% ARPACK is used for solving the generalized eigenvalue problem.

%%

clear all
close all

Nx = 200;

dz = 1/Nx;
r  = 1/dz^2;
z  = 0:dz:1;

% Number of eigenvalues requested for EIGS

nev = 2;

% kv = 2.682;
kv  = linspace(0.5,8,50);
Ra0 = 6000;

%% FDM Coefficients
%  Wiki ref:
needrefs = 1;
if needrefs
    web('https://en.wikipedia.org/wiki/Finite_difference_coefficient#Central_finite_difference')
end

alpha0 = -20;
alpha1 =  15;
alpha2 =  -6;
alpha3 =   1;

beta0  =   6;
beta1  =  -4;
beta2  =   1;

gamma0 =  -2;
gamma1 =   1;

%% Assemble FDM matrices

A1 = hexadiag(alpha3,alpha2,alpha1,alpha0,alpha1,alpha2,alpha3,Nx-1);
A2 = hexadiag(0     , beta2, beta1, beta0, beta1, beta2,     0,Nx-1);
A3 = hexadiag(0     ,     0,gamma1,gamma0,gamma1,     0,     0,Nx-1);
A4 = eye(Nx-1);

%%

figure(1); hold on
xlabel('$\tilde{k}$','fontsize',20,'interpreter','latex');
ylabel('$Ra$','fontsize',20,'interpreter','latex')
axis([0 kv(end) 0 8000]);

%% Iterate on the wavenumber

% Note that the FDM matrices change with k!

for ii = 1:length(kv)
    
    k = kv(ii);
    
    A1(1,1      ) = alpha0 + alpha2 + alpha3*(1/beta2*(2*k^2*2*gamma1*dz^2- 2*beta1)) ;
    A1(1,2      ) = alpha1 - alpha3;
    
    A1(2,1      ) = alpha1 + alpha3;
    
    A1(end,end  ) = alpha0 - alpha2;
    A1(end,end-1) = alpha1 - alpha3;
    
    A1(end-1,end) = alpha1 - alpha3;
    
    
    A2(1,1      ) = beta0  + beta2;
    A2(end,end  ) = beta0  - beta2;
    
    % Assemble operators in form of generalized eigenvalue problem
    
    A =  r^3*A1  - 3*k^2*r^2*A2 + 3*k^4*r*A3 - k^6*A4;
    B = -eye(Nx-1)*k^2;
    
    opts.tol  = 1e-12;
    opts.disp = 1;
    
    % Compute eigenvalues with ARPACK, provided Ra0 as a shift
    
    tic
    [W,D] = eigs(A,B,nev,Ra0,opts);
    toc
    
    Ravec       = diag(D);
    [Rac,pos]   = min(Ravec);

    % Get the full vector for W by appending W(z=0) and W(z=1)

    W = [zeros(1,nev);W;zeros(1,nev)];
    
    plot(k,Rac,'k.'); drawnow;
    
    Ra0    = Rac;
    Rs(ii) = Rac;
    
end

%% Save results

file_name = 'Rac_fdm';
save(file_name,'kv','Rs');
